@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012, 2016, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2012, 2023, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -249,17 +249,16 @@ void RangeCheckEliminator::Visitor::do_ArithmeticOp(ArithmeticOp *ao) {
 
         Bound * bound = _rce->get_bound(y);
         if (bound->has_upper() && bound->has_lower()) {
-          int new_lower = bound->lower() + const_value;
-          jlong new_lowerl = ((jlong)bound->lower()) + const_value;
-          int new_upper = bound->upper() + const_value;
-          jlong new_upperl = ((jlong)bound->upper()) + const_value;
-
-          if (((jlong)new_lower) == new_lowerl && ((jlong)new_upper == new_upperl)) {
-            Bound *newBound = new Bound(new_lower, bound->lower_instr(), new_upper, bound->upper_instr());
-            _bound = newBound;
-          } else {
-            // overflow
+          jint t_lo = bound->lower();
+          jint t_hi = bound->upper();
+          jint new_lower = java_add(t_lo, const_value);
+          jint new_upper = java_add(t_hi, const_value);
+          bool overflow = ((const_value < 0 && (new_lower > t_lo)) ||
+                           (const_value > 0 && (new_upper < t_hi)));
+          if (overflow) {
             _bound = new Bound();
+          } else {
+            _bound = new Bound(new_lower, bound->lower_instr(), new_upper, bound->upper_instr());
           }
         } else {
           _bound = new Bound();
@@ -344,7 +343,12 @@ void RangeCheckEliminator::update_bound(IntegerStack &pushed, Value v, Instructi
 bool RangeCheckEliminator::loop_invariant(BlockBegin *loop_header, Instruction *instruction) {
   assert(loop_header, "Loop header must not be null!");
   if (!instruction) return true;
-  return instruction->dominator_depth() < loop_header->dominator_depth();
+  for (BlockBegin *d = loop_header->dominator(); d != NULL; d = d->dominator()) {
+    if (d == instruction->block()) {
+      return true;
+    }
+  }
+  return false;
 }
 
 // Update bound. Pushes a new bound onto the stack. Tries to do a conjunction with the current bound.
@@ -971,7 +975,7 @@ void RangeCheckEliminator::calc_bounds(BlockBegin *block, BlockBegin *loop_heade
           } else {
             // Has no upper bound
             Instruction *instr = ai->length();
-            if (instr != NULL) instr = ai->array();
+            if (instr == NULL) instr = ai->array();
             update_bound(pushed, ai->index(), Instruction::lss, instr, 0);
           }
         }
@@ -1485,7 +1489,6 @@ void RangeCheckEliminator::Bound::add_assertion(Instruction *instruction, Instru
       NOT_PRODUCT(ao->set_printable_bci(position->printable_bci()));
       result = result->insert_after(ao);
       compare_with = ao;
-      // TODO: Check that add operation does not overflow!
     }
   }
   assert(compare_with != NULL, "You have to compare with something!");
