@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2018, 2023, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -23,16 +23,24 @@
 
 /*
  * @test
- * @bug 8164879
- * @library /lib/testlibrary ../../
- * @summary Verify AES/GCM's limits set in the jdk.tls.keyLimits property
+ * @bug 8164879 8300285
+ * @library ../../
+ * @library /test/lib
+ * @summary Verify AEAD TLS cipher suite limits set in the jdk.tls.keyLimits
+ * property
  * start a new handshake sequence to renegotiate the symmetric key with an
  * SSLSocket connection.  This test verifies the handshake method was called
  * via debugging info.  It does not verify the renegotiation was successful
  * as that is very hard.
  *
- * @run main SSLEngineKeyLimit 0 server AES/GCM/NoPadding keyupdate 1050000
- * @run main SSLEngineKeyLimit 1 client AES/GCM/NoPadding keyupdate 2^22
+ * @run main SSLEngineKeyLimit 0 server TLS_AES_256_GCM_SHA384
+ *      AES/GCM/NoPadding keyupdate 1050000
+ * @run main SSLEngineKeyLimit 1 client TLS_AES_256_GCM_SHA384
+ *      AES/GCM/NoPadding keyupdate 2^22
+ * @run main SSLEngineKeyLimit 0 server TLS_CHACHA20_POLY1305_SHA256
+ *      AES/GCM/NoPadding keyupdate 1050000, ChaCha20-Poly1305 KeyUpdate 1050000
+ * @run main SSLEngineKeyLimit 1 client TLS_CHACHA20_POLY1305_SHA256
+ *      AES/GCM/NoPadding keyupdate 2^22, ChaCha20-Poly1305 KeyUpdate 2^22
  */
 
 /*
@@ -53,9 +61,9 @@ import java.security.KeyStore;
 import java.security.SecureRandom;
 import java.util.Arrays;
 
-import jdk.testlibrary.ProcessTools;
-import jdk.testlibrary.Utils;
-import jdk.testlibrary.OutputAnalyzer;
+import jdk.test.lib.process.ProcessTools;
+import jdk.test.lib.process.OutputAnalyzer;
+import jdk.test.lib.Utils;
 
 public class SSLEngineKeyLimit {
 
@@ -85,7 +93,7 @@ public class SSLEngineKeyLimit {
     }
 
     /**
-     * args should have two values:  server|client, <limit size>
+     * args should have two values:  server|client, cipher suite, <limit size>
      * Prepending 'p' is for internal use only.
      */
     public static void main(String args[]) throws Exception {
@@ -104,7 +112,7 @@ public class SSLEngineKeyLimit {
             File f = new File("keyusage."+ System.nanoTime());
             PrintWriter p = new PrintWriter(f);
             p.write("jdk.tls.keyLimits=");
-            for (int i = 2; i < args.length; i++) {
+            for (int i = 3; i < args.length; i++) {
                 p.write(" "+ args[i]);
             }
             p.close();
@@ -119,10 +127,13 @@ public class SSLEngineKeyLimit {
                     System.getProperty("test.java.opts"));
 
             ProcessBuilder pb = ProcessTools.createJavaProcessBuilder(true,
-                    Utils.addTestJavaOpts("SSLEngineKeyLimit", "p", args[1]));
+                    Utils.addTestJavaOpts("SSLEngineKeyLimit", "p", args[1],
+                            args[2]));
 
             OutputAnalyzer output = ProcessTools.executeProcess(pb);
             try {
+                output.shouldContain(String.format(
+                        "\"cipher suite\"        : \"%s", args[2]));
                 if (expectedFail) {
                     output.shouldNotContain("KeyUpdate: write key updated");
                     output.shouldNotContain("KeyUpdate: read key updated");
@@ -170,9 +181,10 @@ public class SSLEngineKeyLimit {
         cTos.clear();
         sToc.clear();
 
-        Thread ts = new Thread(serverwrite ? new Client() : new Server());
+        Thread ts = new Thread(serverwrite ? new Client() :
+                new Server(args[2]));
         ts.start();
-        (serverwrite ? new Server() : new Client()).run();
+        (serverwrite ? new Server(args[2]) : new Client()).run();
         ts.interrupt();
         ts.join();
     }
@@ -416,11 +428,14 @@ public class SSLEngineKeyLimit {
     }
 
     static class Server extends SSLEngineKeyLimit implements Runnable {
-        Server() throws Exception {
+        Server(String cipherSuite) throws Exception {
             super();
             eng = initContext().createSSLEngine();
             eng.setUseClientMode(false);
             eng.setNeedClientAuth(true);
+            if (cipherSuite != null && cipherSuite.length() > 0) {
+                eng.setEnabledCipherSuites(new String[] { cipherSuite });
+            }
         }
 
         public void run() {
