@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012, 2015, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2012, 2018, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -1252,6 +1252,11 @@ public class ClassfileInspector {
             return null;
         }
 
+        @Override
+          public Void visitNestHost(NestHost_attribute attr, T p) {
+            return null;
+        }
+
         @Override
         public Void visitMethodParameters(MethodParameters_attribute attr, T p) {
             return null;
@@ -1287,6 +1292,11 @@ public class ClassfileInspector {
             return null;
         }
 
+        @Override
+        public Void visitNestMembers(NestMembers_attribute attr, T p) {
+            return null;
+        }
+
         @Override
         public Void visitRuntimeInvisibleAnnotations(RuntimeInvisibleAnnotations_attribute attr, T p) {
             return null;
